package testthreadid.handlers;

import org.eclipse.cdt.dsf.datamodel.IDMContext;
import org.eclipse.cdt.dsf.debug.service.IBreakpoints.IBreakpointDMContext;
import org.eclipse.cdt.dsf.debug.service.IBreakpointsExtension.IBreakpointHitDMEvent;
import org.eclipse.cdt.dsf.debug.service.IProcesses.IThreadDMContext;
import org.eclipse.cdt.dsf.debug.service.IRunControl.IContainerSuspendedDMEvent;
import org.eclipse.cdt.dsf.debug.service.IRunControl.IExecutionDMContext;
import org.eclipse.cdt.dsf.gdb.launching.GdbLaunch;
import org.eclipse.cdt.dsf.service.DsfServiceEventHandler;
import org.eclipse.cdt.dsf.service.DsfSession;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchListener;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.handlers.HandlerUtil;

public class SampleHandler extends AbstractHandler implements ILaunchListener {

	private DsfSession m_session = null;
	
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		
		
		IWorkbenchWindow window = HandlerUtil.getActiveWorkbenchWindowChecked(event);
		MessageDialog.openInformation(
				window.getShell(),
				"TestThreadID",
				"Test Thread ID - listeners attached - ready to launch debug session");
		ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
		manager.addLaunchListener(this);
		return null;
	}

	@Override
	public void launchRemoved(ILaunch launch) {}

	@Override
	public void launchAdded(ILaunch launch) {
		if (launch instanceof GdbLaunch) {
			GdbLaunch gdbLaunch = (GdbLaunch)launch;
			this.m_session = gdbLaunch.getSession();
			this.m_session.addServiceEventListener(this, null);
		}		
	}
	
	/**
	 * Search for the first {@link IThreadDMContext}. Perform the search only in the first instance in the array.
	 * @param triggeringContexts The array to search.
	 * @return the first found {@link IThreadDMContext} or null.
	 */
	private IThreadDMContext getFirstTriggeringThreadDMContext(IExecutionDMContext[] triggeringContexts) {
		IThreadDMContext retval = null;
		if ((triggeringContexts != null) && (triggeringContexts.length > 0)) {
			IExecutionDMContext firstTriggeringContext = triggeringContexts[0];		
			if (firstTriggeringContext instanceof IThreadDMContext) {
				retval = (IThreadDMContext)firstTriggeringContext;
			} else {
				//Search its parents
				IDMContext[] parents = firstTriggeringContext.getParents();				
				for (int index = 0; (retval == null) && (index < parents.length); ++index) {
					IDMContext DMContextParent = parents[index];
					if (DMContextParent instanceof IThreadDMContext) {
						retval = (IThreadDMContext)DMContextParent;
					}
				}
			}
		}
		return retval;
	}	
	
	@DsfServiceEventHandler
	public void eventReceived(IContainerSuspendedDMEvent event) {
		IExecutionDMContext DMContext =  event.getDMContext();
				
		if (event instanceof IBreakpointHitDMEvent) {
			IThreadDMContext firstTriggeringThreadDMContext = getFirstTriggeringThreadDMContext(event.getTriggeringContexts());			
			try {				 
				this.m_session.getExecutor().submit(new GetThreadIDRunnable(DMContext, firstTriggeringThreadDMContext, this.m_session.getExecutor()));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}		
	}

	@Override
	public void launchChanged(ILaunch launch) {}
}
